@@ -10,7 +10,7 @@ use super::{
     session::{Session, SessionSendError, SessionSendErrorType},
     socket_forwarder::SocketForwarder,
 };
-use crate::event::EventTypeFlags;
+use crate::{event::EventTypeFlags, shard::tls::TlsContainer};
 use serde::{Deserialize, Serialize};
 use std::{
     borrow::Cow,
@@ -35,9 +35,11 @@ use twilight_model::gateway::{
         DispatchEvent, Event, GatewayEvent, GatewayEventDeserializer,
     },
     payload::{
-        identify::{Identify, IdentifyInfo, IdentifyProperties},
-        resume::Resume,
-        Ready,
+        incoming::Ready,
+        outgoing::{
+            identify::{Identify, IdentifyInfo, IdentifyProperties},
+            resume::Resume,
+        },
     },
     Intents, OpCode,
 };
@@ -312,7 +314,7 @@ impl ShardProcessor {
             gateway: url.clone(),
             shard_id: config.shard()[0],
         }));
-        let stream = Self::connect(&url).await?;
+        let stream = Self::connect(&url, config.tls.as_ref()).await?;
         let (forwarder, rx, tx) = SocketForwarder::new(stream);
         tokio::spawn(async move {
             forwarder.run().await;
@@ -436,7 +438,7 @@ impl ShardProcessor {
                     });
                 };
 
-            // We can do a few little optimisation tricks here. For the
+            // We can do a few little optimization tricks here. For the
             // "heartbeat ack" and "reconnect" opcodes we can construct
             // the gateway events without needing to go through a serde
             // context.
@@ -840,8 +842,10 @@ impl ShardProcessor {
         Ok(())
     }
 
-    async fn connect(url: &str) -> Result<ShardStream, ConnectingError> {
-        #[allow(rust_2021_incompatible_closure_captures)]
+    async fn connect(
+        url: &str,
+        tls: Option<&TlsContainer>,
+    ) -> Result<ShardStream, ConnectingError> {
         let url = Url::parse(url).map_err(|source| ConnectingError {
             kind: ConnectingErrorType::ParsingUrl {
                 url: url.to_owned(),
@@ -861,12 +865,16 @@ impl ShardProcessor {
             max_send_queue: None,
         };
 
-        let (stream, _) = tokio_tungstenite::connect_async_with_config(url, Some(config))
-            .await
-            .map_err(|source| ConnectingError {
-                kind: ConnectingErrorType::Establishing,
-                source: Some(Box::new(source)),
-            })?;
+        let (stream, _) = tokio_tungstenite::connect_async_tls_with_config(
+            url,
+            Some(config),
+            tls.map(TlsContainer::connector),
+        )
+        .await
+        .map_err(|source| ConnectingError {
+            kind: ConnectingErrorType::Establishing,
+            source: Some(Box::new(source)),
+        })?;
 
         #[cfg(feature = "tracing")]
         tracing::debug!("Shook hands with remote");
@@ -926,7 +934,7 @@ impl ShardProcessor {
                 shard_id: self.config.shard()[0],
             }));
 
-            let stream = match Self::connect(&self.url).await {
+            let stream = match Self::connect(&self.url, self.config.tls.as_ref()).await {
                 Ok(s) => s,
                 Err(_source) => {
                     #[cfg(feature = "tracing")]
@@ -995,7 +1003,7 @@ impl ShardProcessor {
             shard_id: self.config.shard()[0],
         }));
 
-        let stream = Self::connect(&self.url).await?;
+        let stream = Self::connect(&self.url, self.config.tls.as_ref()).await?;
 
         self.set_session(stream, Stage::Resuming);
 
